@@ -1,4 +1,4 @@
-#!/usr/local/bin/perl -l
+#!/usr/local/bin/perl 
 
 # see http://www.compapp.dcu.ie/~alex/LOGIC/monkey.html
 # This is the classic Monkey/Banana problem
@@ -32,12 +32,13 @@ perform(walk(P1,P2),
 getfood(state(Bogus1,Bogus2,Bogus3,has)).
 
 getfood(S1) :- perform(Act, S1, S2),
-              nl, print(S1), print(Act),
+              nl, print('In '), print(S1), print(' try '), print(Act), nl,
               getfood(S2).
 END_PROLOG
 
 my $parser = Parser->new("getfood(state(atdoor,atwindow,onfloor,hasnot)).");
 my $query  = Term->new($parser);
 my $engine = Engine->new($query,$database);
+Engine->formatted(1);
 
 print $engine->results;
