@@ -1,6 +1,6 @@
 package AI::Prolog::Engine;
-
-$VERSION = '0.01';
+$REVISION = '$Id: Engine.pm,v 1.4 2005/01/29 16:44:47 ovid Exp $';
+$VERSION = '0.02';
 use strict;
 use warnings;
 
@@ -23,11 +23,37 @@ use constant OnceMark => 'OnceMark';
 # This governs whether tracing is done
 my $TRACE;
 sub trace {
-    my ($self, $trace) = @_;
-    $TRACE = $trace if defined $trace;
+    my $class = shift;
+    if (@_) {
+        $TRACE = shift;
+        return $class;
+    }
     return $TRACE;
 }
 
+my $FORMATTED = 1;
+sub formatted {
+    my $self = shift;
+    if (@_) {
+        $FORMATTED = shift;
+        return $self;
+    }
+    return $FORMATTED;
+}
+
+my $RAW_RESULTS;
+sub raw_results {
+    my $self = shift;
+    if (@_) {
+        $RAW_RESULTS = shift;
+        if ($RAW_RESULTS) {
+            $self->formatted(0);
+        }
+        return $self;
+    }
+    return $RAW_RESULTS;
+}
+
 sub new {
     my ($class, $term, $prog) = @_;
     Term->internalparse(1); # enable underscore as first character of term
@@ -133,9 +159,16 @@ sub _run {
 
     while (1) {
         unless ($self->{_goal}) {
-            # here's where we set the time
-            return $self->_call->to_string;
-            #$self->more;
+            # we've succeeded.  return results
+            if ($self->formatted) {
+                return $self->_call->to_string;
+            }
+            else {
+                my @results = $self->_call->to_data;
+                return $self->raw_results
+                    ? $results[1]
+                    : $results[0];
+            }
         }
         
         unless (defined $self->_goal && $self->{_goal}->term) {
@@ -289,29 +322,81 @@ C<AI::Prolog::Engine> is a Warren Abstract Machine (WAM) based Prolog engine.
 
 The C<new()> function actually bootstraps some Prolog code onto your program to
 give you access to the built in predicates listed in the
-L<AI::Prolog|AI::Prolog> documentation.
+L<AI::Prolog::Builtins|AI::Prolog::Builtins> documentation.
 
-See L<AI::Prolog|AI::Prolog> for more information.
+This documentation is provided for completeness.  You probably want to use
+L<AI::Prolog|AI::Prolog>.
 
 =head1 CLASS METHODS
 
 =head2 C<new($query, $database)>
 
 This creates a new Prolog engine.  The first argument must be of type
-C<AI::Prolog::Term> and the second my be a database created by
+C<AI::Prolog::Term> and the second must be a database created by
 C<AI::Prolog::Parser::consult>.
 
  my $database = Parser->consult($some_prolog_program);
  my $query    = Term->new('steals(badguy, X).');
  my $engine   = Engine->new($query, $database);
+ Engine->formatted(1);
  while (my $results = $engine->results) {
     print $results, $/;
  }
 
+The need to have a query at the same time you're instantiating the engine is a
+bit of a drawback based upon the original W-Prolog work.  I will likely remove
+this drawback in the future.
+
+=head2 C<formatted([$boolean])>
+
+The default value of C<formatted> is true.  This method, if passed a true
+value, will cause C<results> to return a nicely formatted string representing
+the output of the program.  This string will loosely correspond with the
+expected output of a Prolog program.
+
+If false, all calls to C<result> will return Perl data structures instead of
+nicely formatted output.
+
+If called with no arguments, this method returns the current C<formatted>
+value.
+
+ Engine->formatted(1); # turn on formatting
+ Engine->formatted(0); # turn off formatting (default)
+ 
+ if (Engine->formatted) {
+     # test if formatting is enabled
+ }
+
+B<Note>: if you choose to use the L<AI::Prolog|AI::Prolog> interface instead of
+interacting directly with this class, that interface will set C<formatted> to
+false.  You will have to set it back in your code if you do not wish this
+behavior:
+
+ use AI::Prolog;
+ my $logic = AI::Prolog->new($prog_text);
+ $logic->query($query_text);
+ AI::Logic::Engine->formatted(1); # if you want formatted to true
+ while (my $results = $logic->results) {
+    print "$results\n";
+ }
+
+=head2 C<raw_results([$boolean])>
+
+The default value of C<raw_results> is false.  Setting this property to a true
+value automatically sets C<formatted> to false.  C<results> will return the raw
+data structures generated by questions when this property is true.
+ 
+ Engine->raw_results(1); # turn on raw results
+ Engine->raw_results(0); # turn off raw results (default)
+ 
+ if (Engine->raw_results) {
+     # test if raw results is enabled
+ }
+
 =head2 C<trace($boolean)>
 
-Set this to a true value to turn on tracing.  This will trace through the engine's
-goal satisfaction process while it's running.  This is very slow.
+Set this to a true value to turn on tracing.  This will trace through the
+engine's goal satisfaction process while it's running.  This is very slow.
 
  Engine->trace(1); # turn on tracing
  Engine->trace(0); # turn off tracing
@@ -320,21 +405,42 @@ goal satisfaction process while it's running.  This is very slow.
 
 =head2 C<results()>
 
-This method will return the results from the last run query, one result at a time.
-It will return false when there are no more results.
+This method will return the results from the last run query, one result at a
+time.  It will return false when there are no more results.  If C<formatted> is
+true, it will return a string representation of those results:
 
  while (my $results = $engine->results) {
-    print $results, $/;
+    print "$results\n";
+ }
+
+If C<formatted> is false, C<$results> will be an object with methods matching
+the variables in the query.  Call those methods to access the variables:
+
+ AI::Prolog::Engine->formatted(0);
+ $engine->query('steals(badguy, STUFF, VICTIM).');
+ while (my $r = $engine->results) {
+     printf "badguy steals %s from %s\n", $r->STUFF, $r->VICTIM;
+ }
+
+If necessary, you can get access to the full, raw results by setting
+C<raw_results> to true.  In this mode, the results are returned as an array
+reference with the functor as the first element and an additional element for
+each term.  Lists are represented as array references.
+
+ AI::Prolog::Engine->raw_results(1);
+ $engine->query('steals(badguy, STUFF, VICTIM).');
+ while (my $r = $engine->results) {
+    # do stuff with $r in the form:
+    # ['steals', 'badguy', $STUFF, $VICTIM]
  }
 
 =head2 C<query($query)>
 
-If you already have an engine object instantiated, call the C<query()>
-method for subsequent queries.  Internally, when calling C<new()>, the
-engine bootstraps a set of Prolog predicates to provide the built ins.
-However, this process is slow.  Subsequent queries to the same engine
-with the C<query()> method can double the speed of your program.
-
+If you already have an engine object instantiated, call the C<query()> method
+for subsequent queries.  Internally, when calling C<new()>, the engine
+bootstraps a set of Prolog predicates to provide the built ins.  However, this
+process is slow.  Subsequent queries to the same engine with the C<query()>
+method can double the speed of your program.
  
  my $engine   = Engine->new($query, $database);
  while (my $results = $engine->results) {
@@ -346,5 +452,30 @@ with the C<query()> method can double the speed of your program.
     print $results, $/;
  }
 
-=cut
+=head1 BUGS
 
+A query using C<[HEAD|TAIL]> syntax does not bind properly with the C<TAIL>
+variable when returning a result object.  This is due to a bug in
+L<AI::Prolog::Term|AI::Prolog::Term>'s C<_to_data()> method.
+
+Solutions:  use C<raw_results> and parse the resulting data structure yourself
+or restructure you query to not require the C<[HEAD|TAIL]> syntax.
+
+=head1 AUTHOR
+
+Curtis "Ovid" Poe, E<lt>moc tod oohay ta eop_divo_sitrucE<gt>
+
+Reverse the name to email me.
+
+This work is based on W-Prolog, http://goanna.cs.rmit.edu.au/~winikoff/wp/,
+by Dr. Michael Winikoff.  Many thanks to Dr. Winikoff for granting me
+permission to port this.
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2005 by Curtis "Ovid" Poe
+
+This library is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself. 
+
+=cut
