@@ -1,6 +1,7 @@
 package AI::Prolog::TermList;
+$REVISION = '$Id: TermList.pm,v 1.4 2005/01/29 16:44:47 ovid Exp $';
 
-$VERSION = 0.01;
+$VERSION = 0.02;
 
 use strict;
 use warnings;
@@ -126,10 +127,35 @@ AI::Prolog::TermList - Create lists of Prolog Terms.
 
 =head1 SYNOPSIS
 
-No need for you.  You should never be seeing this.
+No user serviceable parts inside.  You should never be seeing this.
 
 =head1 DESCRIPTION
 
 See L<AI::Prolog|AI::Prolog> for more information.  If you must know more,
 there are plenty of comments sprinkled through the code.
- 
+
+=head1 SEE ALSO
+
+W-Prolog:  L<http://goanna.cs.rmit.edu.au/~winikoff/wp/>
+
+Michael BartE<225>k's online guide to programming Prolog:
+L<http://kti.ms.mff.cuni.cz/~bartak/prolog/index.html>
+
+=head1 AUTHOR
+
+Curtis "Ovid" Poe, E<lt>moc tod oohay ta eop_divo_sitrucE<gt>
+
+Reverse the name to email me.
+
+This work is based on W-Prolog, L<http://goanna.cs.rmit.edu.au/~winikoff/wp/>,
+by Dr. Michael Winikoff.  Many thanks to Dr. Winikoff for granting me
+permission to port this.
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2005 by Curtis "Ovid" Poe
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=cut
